
/**
 * Copyright 2012 Clint Combs
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.newhart

import java.util.{Date => JDate}
import com.mongodb.casbah.Imports._

/**
 * An Entry in a newhart audit store.
 */
case class Entry(
  origin:String,
  originKey:String,
  originKeyType:String,
  originHostname:String,
  criticality:Criticality.Value,
  auditTS:JDate,
  createTS:JDate, /* TODO: update to Option[JDate] */
  updateTS:JDate, /* TODO: update to Option[JDate] */
  msg:String,
  data:Map[String,Any],
  errors:Seq[Error] = List(),
  warnings:Seq[Warning] = List(),
  labels:Set[String] = Set()) {

  require((origin != null) && (origin.trim.length > 0), "origin must not be empty")
  require((originKey != null) && (originKey.trim.length > 0), "originKey must not be empty")
  require((originKeyType != null) && (originKeyType.trim.length > 0), "originKeyType must not be empty")
  require((criticality != null), "criticality must not be null")
  require(auditTS != null, "auditTS must not be null")
  require(updateTS != null, "updateTS must not be null")
  require(msg != null, "msg must not be null")
}

/**
 * The companion object to the newhart Entry class.  Provides easy access to the
 * Entry Builder class.
 */
object Entry {
  def now() = new JDate()

  def builder() =
    Builder(origin="", originKey="", originKeyType="", originHostname="", criticality=Criticality.major, auditTS=now,
      createTS=now, updateTS=now, msg="")

  case class Builder(
    var origin:String,
    var originKey:String,
    var originKeyType:String,
    var originHostname:String,
    var criticality:Criticality.Value,
    var auditTS:JDate,
    var createTS:JDate, /* TODO: update to Option[JDate] */
    var updateTS:JDate, /* TODO: update to Option[JDate] */
    var msg:String,
    data:scala.collection.mutable.Map[String,Any] = scala.collection.mutable.Map[String,Any](),
    errors:scala.collection.mutable.ListBuffer[Error] = scala.collection.mutable.ListBuffer(),
    warnings:scala.collection.mutable.ListBuffer[Warning] = scala.collection.mutable.ListBuffer(),
    labels:scala.collection.mutable.SetBuilder[String, Set[String]] =
      new scala.collection.mutable.SetBuilder[String, Set[String]](Set())) {

    def result() = {
      Entry(origin=origin, originKey=originKey, originKeyType=originKeyType,
        originHostname=replaceEmptyString(originHostname, defaultOriginHostname()),
        criticality=criticality, auditTS=auditTS, createTS=createTS, updateTS=updateTS,
        msg=msg, data=data.toMap, errors=errors.toList, warnings=warnings.toList, labels=labels.result)
    }

    /**
     * If s is null or empty, replace it with the result of f. The function f
     * is called by value in order to avoid calling f repeatedly.  This may
     * change in the future to call by name (f: => String) if warranted.
     */
    private def replaceEmptyString(s:String, f:String) = {
      if ((s == null) || (s.trim() == "")) f else s
    }

    require(origin != null, "origin must not be null")
    require(originKey != null, "originKey must not be null")
    require(originKeyType != null, "originKeyType must not be null")
    require(criticality != null, "criticality must not be null")
    require(auditTS != null, "auditTS must not be null")
    require(updateTS != null, "updateTS must not be null")
    require(msg != null, "msg must not be null")
  }
}

object EntryUtil {
  def simple() =
    Entry(origin = "origin", originKey = "originKey", originKeyType = "originKeyType", originHostname="originHostname",
      criticality = Criticality.major, auditTS = new JDate(), createTS = new JDate(), updateTS = new JDate(),
      msg = "a message", data = Map())
}

/** An Error component of an Entry. */
case class Error(msg:Option[String], code:Option[String]=None, stackTrace:Option[String]=None) {
  require(msg != None || code != None || stackTrace != None)
}

/** A Warning component of an Entry. */
case class Warning(msg:Option[String], code:Option[String]=None, stackTrace:Option[String]=None) {
  require(msg != None || code != None || stackTrace != None)
}

/** The criticality of an Entry. */
object Criticality extends Enumeration {
  val minor, major, critical = Value
}

// TODO: The concept of MIME/Java type might be useful in another context; Value is being
// replaced by MongoDBObject, but maybe it would make sense to have a convention that a
// data object can be set up to have a MIME or Java type and provide hooks to convert
// to/from these types
//
//case class Value(text:String, mimeType:Option[String]=None, javaType:Option[String]=None)
